#!/usr/bin/perl
#
# SDPL - Sequential Discontinuos Program Launcher
#
# Copyright (C) 2011 Jordi "Txor" Casas Ríos <txorlings@gmail.com> 
# 
# sdpl is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for further
# details, currently published at http://www.gnu.org/licenses/gpl.txt or in the
# gpl.txt in the ["legal" folder] of this distribution.
# 
# You should have received a copy of the GNU General Public License along with
# this program; If not, see <http://www.gnu.org/licenses/>
# 
# Authors: Jordi "Txor" Casas Ríos (txorlings@gmail.com)
# 

# Modules
#--------
use warnings;
use strict;
use Getopt::Long;

# Global variables
#-----------------

# Path files
my $file_index;
my $file_programs;
my @config_sdpl;
my @program_list;

# Specific values
my $num_progs;
my $index;

# Subroutines
#------------

# main is the start point, see if there are options or make the standard execution otherwise.
sub main {
	&usage if ! GetOptions(
			'reset|r'	=> \&reset,
			'index|i'	=> \&index,
			'total|t'	=> \&total,
			'list|l'	=> \&list,
			'help|h'	=> \&usage,
			'<>'		=> \&usage
		);
	&std_run;
}

# usage shows sdpl options and exits.
sub usage {
	print "SDPL - Sequential Discontinuos Program Launcher\n";
	print "Copyright (C) 2011 Jordi \"Txor\" Casas Ríos <txorlings\@gmail.com>\n";
	print "\n";
	print "Usage: sdpl [options]\n";
	print "Runs configured programs, one per each SDPL execution.\n";
	print "\n";
	print "options:\n";
	print "  -r, --reset:\t\tResets the program counter, the next execution will start the first program.\n";
	print "  -i, --index:\t\tPrints the number of the current program to be executed (ex: 1 for the first).\n";
	print "  -t, --total:\t\tPrints the total number of current configured programs.\n";
	print "  -l, --list:\t\tPrints the command list.\n";
	print "  -h, --help:\t\tShows this help.\n";
	print "\n";
	print "files:\n";
	print "  /etc/sdpl/sdpl.conf\n";
	print "\n";
	exit 0;
}

#  load_config searches /etc/sdpl/sdpl.conf for CURRENT and PROGRAMS config variables.
#              Loads them and checks if all exists. Returns on success, die on fail.
sub load_config {
	my $b_curr;
	my $b_prog;
	$b_curr = $b_prog = 0;
	foreach (@config_sdpl) {
		if (($b_curr == 0) && (/^CURRENT=(.*)/)) {
			$file_index=$1;
			$b_curr = 1;
		}
		elsif (($b_prog == 0) && (/^PROGRAMS=(.*)/)) {
			$file_programs=$1;
			$b_prog = 1;
		}
	}
	if (($b_curr == 0) || ($b_prog == 0)) {
		die "ERROR: Configuration load failed: /etc/sdpl/sdpl.conf";
	}
}

# read_config_file fills @config_sdpl with /etc/sdpl/sdpl.conf file content.
sub read_config_file {
	open(CONFIG_SDPL, "<", "/etc/sdpl/sdpl.conf") or die "ERROR: Cannot open config file: $!";
	@config_sdpl = <CONFIG_SDPL>;
	close(CONFIG_SDPL);
	load_config;
}

# load_index loads $index with the $file_index value if the file exists, otherwise the file is created with a 0 value and $index becomes 0.
sub load_index {
	if (-e $file_index) {
		open(FILE, "<", "$file_index") or die "ERROR: Cannot open file $file_index: $!";
		$index = <FILE>; 
		close(FILE);
	}
	else {
		&reset_index;
		$index = 0; 
	}
}

# load_programs fills @program_list with the contents of $file_programs file and $num_progs with the number of lines.
sub load_programs {
	unless (open(FILE, "<", "$file_programs")) {
		print "INFO: Perhaps sdpl needs to be configured.\n";
		die "ERROR: Cannot open file $file_programs: $!";
	}
	@program_list = <FILE>;
	close(FILE);
	foreach (@program_list) {
		$_ =~ s/\r|\n//g;
	}
	$num_progs = @program_list;
}

# check_index checks that $index is a valid value
sub check_index {
	if ($index !~ /^[0-9]+$/) { die "ERROR: Invalid index value.\n"; }
	if ($index >= $num_progs) { exit 0; }
}

# check_program checks if the parameter is a valid command after
#               removing possible environment variables and parameters.
#               returns 0 if the command isn't executable, returns the command otherwise.
sub check_program {
	my $line = shift;
	$line =~ s/^(?:.*=[^ ]*)* (.*)/$1/;
	$line =~ s/^([^ ]*).*/$1/;
	return 0 if ! -x $line;
	return $line;
}

# run_program tries to run the $program_list[N] file where N is the subroutine parameter.
sub run_program {
	system $program_list[$_[0]];
	if ( $? != 0 ) {
		die "ERROR: Execution of $program_list[$_[0]] falied: $!\n";
	}
}

# save_index writes the current value of $index to $file_index file.
sub save_index {
	open (FILE, ">" ,"$file_index");
	print FILE $index;
	close (FILE);
}

# reset_index writes a 0 on $file_index file.
sub reset_index {
	open (FILE, ">" ,"$file_index") or die "ERROR: Cannot open file: $!";
	print FILE "0";
	close (FILE); 
}

# reset launches the reset of the file index and exits.
sub reset {
	&read_config_file;
	&load_config;
	&reset_index;
	exit 0;
}

# index prints the number of the current program to be executed and exits.
sub index {
	&read_config_file;
	&load_config;
	&load_index;
	print ++$index."\n";
	exit 0;
}

# total prints the total number of current configured programs.
sub total {
	&read_config_file;
	&load_config;
	&load_programs;
	print "$num_progs\n";
	exit 0;
}

# list Prints a list of configured programs.
sub list {
	&read_config_file;
	&load_config;
	&load_index;
	&load_programs;
	my $i = 0;
	foreach (@program_list) {
		print $i+1;
		print "\t";
		&check_program($program_list[$i]) ? print "Ok\t" : print "Ko\t";
		print "$_\n";
		$i++;
	}
	exit 0;
}

# std_run launches the normal execution.
sub std_run {
	&read_config_file;
	&load_config;
	&load_index;
	&load_programs;
	&check_index;
	die "ERROR: Invalid program $program_list[$index]: $!.\n" if ! &check_program($program_list[$index]);
	my $tmp_index = $index;
	$index++;
	&save_index;
	&run_program($tmp_index);
	exit 0;
}

main();

